# Favicon - Adds favicons to comments and trackbacks
# Henrik Gemal - http://gemal.dk/mt/favicon.html
# If you modify this code please let me know and I'll try to incorporate the changes into the next version

package MT::Plugin::Favicon;

use strict;
use warnings;
use MT::Plugin;
use MT::Template::Context;
use MT::PluginData;
use HTTP::Request;
use URI::URL;
use LWP::UserAgent;

use vars qw($VERSION $DEBUG);
$VERSION = "1.4";
$DEBUG = 0;

my $plugin = MT::Plugin->new({
	name => "Favicon",
	description => "Adds favicons to Comments and Trackbacks",
	doc_link => "http://gemal.dk/mt/favicon.html",
	author_name => "Henrik Gemal",
	author_link => "http://gemal.dk/",
	version => $VERSION,
	config_link => "favicon.cgi",
	config_template => \&config_tmpl,
	settings => new MT::PluginSettings([
		["favicon_cache_time", { Default => 604800 }] # cache pages for a week per default
	]),
});
MT->add_plugin($plugin);

MT::Template::Context->add_tag("Favicon" => \&favicon);
MT::Template::Context->add_conditional_tag("FaviconIfAvailable" =>\&favicon);
MT::Template::Context->add_tag("FaviconComment" => \&favicon_comment);
MT::Template::Context->add_conditional_tag("FaviconCommentIfAvailable" =>\&favicon_comment);
MT::Template::Context->add_tag("FaviconTrackback" => \&favicon_trackback);
MT::Template::Context->add_conditional_tag("FaviconTrackbackIfAvailable" =>\&favicon_trackback);

sub config {
	my $config = {};
	if ($plugin) {
		require MT::Request;
		my ($scope) = (@_);
		$config = MT::Request->instance->cache("favicon_config_" . $scope);
		if (!$config) {
			$config = $plugin->get_config_hash($scope);
			MT::Request->instance->cache("favicon_config_" . $scope, $config);
		}
	}
	return $config;
}

sub config_tmpl {
	my $tmpl = <<'EOT';
	<div class="setting">
		<div class="label">
			<label for="cache_time">Cache favicons:</label>
		</div>
		<div class="field">
			<p><input name="favicon_cache_time" id="favicon_cache_time" size="6" maxlength="6" value="<TMPL_VAR NAME=favicon_cache_time ESCAPE=HTML>" /> seconds</p>
		</div>
	</div>
EOT
}

sub favicon_debug {
	my($txt, $force) = @_;
	if ($txt ne "") {
		print STDERR "MTFavicon: $txt\n" if ($DEBUG || $force);
	}
}

sub favicon {
	my($ctx, $args) = @_;
	my $c;
	my $u;
	if ($c = $ctx->stash("ping")) {
		favicon_debug("Trackback in MTFavicon");
		$u = $c->source_url;
	} elsif ($c = $ctx->stash("comment")) {
		favicon_debug("Comment in MTFavicon");
		$u = $c->url;
		favicon_debug("URL for comment ID <" . $c->id . "> is <$u>");
	} else {
		favicon_debug("Error: MTFavicon called without a comment or trackback in context");
		return $ctx->error("Error: MTFavicon called without a comment or trackback in context");
	}
	return favicon_return($u, $ctx, $args);
}

sub favicon_comment {
	my($ctx, $args) = @_;
	my $c = $ctx->stash("comment") || return $ctx->error("Error: MTFaviconComment called without a comment in context");
	favicon_debug("Comment in FaviconComment");
	my $u = $c->url;
	favicon_debug("URL for comment ID <" . $c->id . "> is <$u>");
	return favicon_return($u, $ctx, $args);
}

sub favicon_trackback {
	my($ctx, $args) = @_;
	my $c = $ctx->stash("ping") || return $ctx->error("Error: MTFaviconTrackback called without a trackback in context");
	favicon_debug("Trackback in FaviconTrackback");
	my $u = $c->source_url;
	favicon_debug("URL for trackback ID <" . $c->id . "> is <$u>");
	return favicon_return($u, $ctx, $args);
}

sub favicon_return {
	my($url, $ctx, $args) = @_;
	my $ret = favicon_get($url, $ctx);
	if ($ret && exists $args->{full} && $args->{full}) {
		$ret = favicon_full($ret, $url);
	}
	return $ret;
}

sub favicon_request {
	my($url) = @_;

	# setup useragent
	my $ua = new LWP::UserAgent(timeout => 10);
	$ua->agent("Mozilla/5.0 MTFavicon/$VERSION http://gemal.dk/");

	# do the request
	my $timer = time;
	my $req = HTTP::Request->new(GET => $url);
	my $res = $ua->request($req);
	my $timernew = time;
	favicon_debug("Getting <$url> took <" . ($timernew - $timer) . "> seconds");

	return $res;
}

sub favicon_checkdate {
	my($date, $datecache) = @_;

	my $found = 0;
	my $datediff = time - $date;
	if ($datediff && $datediff > $datecache) {
		favicon_debug("Page date diff <$datediff> older than <$datecache>");
		$found = 1;
	} else {
		favicon_debug("Page date diff <$datediff> not older than <$datecache>");
	}
	return $found;
}

sub favicon_validimage {
	my($res, $url) = @_;

	my $found = 0;
	if ($res->is_success) {
		if ($res->content && $res->content !~ /<(html|head|title|body|table)/i) {
			favicon_debug("Favicon found at <$url>");
			$found = 1;
		} else {
			favicon_debug("Favicon at <$url> isn't a valid favicon");
		}
	} else {
		favicon_debug("Favicon at <$url> not found");
	}
	return $found;
}

sub favicon_get {
	my($url, $ctx) = @_;

	my $icon = "";
	my $favicon;
	my $favicon_additional;

	my $page_cache_time = config("blog:" . $ctx->stash("blog")->id)->{favicon_cache_time};

	if ($url) {

		# remove weird http from https URL
		if ($url =~ /^http:\/\/(https:\/\/.*$)/) {
			#favicon_debug("Strip weird http from https URL. URL was <$url>. URL is now <$1>");
			$url = $1;
		}

		# is the hostname valid? it must contain a dot
		my $tmp = new URI::URL($url);
		my $tmp_host;
		# try to get the hostname. it might fail if the url looks like blabla://test.test
		eval {
			$tmp_host = $tmp->host;
		};
		if ($@) {
			favicon_debug("Error: URL <$url> cant be parsed due to: " . $@);
			return "";
		}
		if ($tmp && $tmp_host && $tmp_host =~ /\.(\w+)/ && $tmp_host !~ /^127\.0\.0\.1|localhost$/i) {

			# strip stuff like #hep from URL
			$favicon->{url} = $tmp->scheme . ":" . $tmp->opaque;
			favicon_debug("URL now <" . $favicon->{url} . "> and before <$url>") if $favicon->{url} != $url;

			# should we look the favicon?
			my $do_request = 1;

			# is there anything in the PluginData?
			if (my $data = MT::PluginData->load({plugin => "Favicon", key => $favicon->{url}})) {
				favicon_debug("<$favicon->{url}> found in PluginData");
				if (!exists ${$data->data}->{favicon}) {
					favicon_debug("But has no favicon");
				} else {
					$favicon->{favicon} = ${$data->data}->{favicon};
				}
				if (!favicon_checkdate(${$data->data}->{date}, $page_cache_time)) {
					$do_request = 0;
				}
			}

			# Now do the hard work and save it
			if ($do_request) {
				my $res = favicon_request($favicon->{url});

				# Look for a header like these
				# <link rel="shortcut icon" href="http://gemal.dk/favicon.ico" />
				# <link rel="icon" type="image/png" href="/images/sitelicon.png" />
				# <link rel="icon" href="mozilla-16.png" type="image/png">
				if ($res->content && $res->content =~ /("|')(shortcut )?icon("|').*?href=("|')(.*?)("|')/i) {
					favicon_debug("Shortcut header found <$5>");
					my $temp = URI::URL::url($5, $res->base)->abs->as_string;

					# Check and see if the favicon really exists
					$res = favicon_request($temp);
					if (favicon_validimage($res, $temp)) {
						$favicon->{favicon} = $temp;
					}
				} else {

					$do_request = 1;
					# need to do some additional work since no shortcut header was found
					favicon_debug("Shortcut header not found");
					my $temp = $tmp->scheme . "://" . ($tmp->port != "80" ? $tmp->host_port : $tmp->host) . "/";
					if (my $data = MT::PluginData->load({plugin => "Favicon", key => $temp})) {
						favicon_debug("<$temp> found in PluginData");
						if (exists ${$data->data}->{favicon}) {
							$favicon->{favicon} = ${$data->data}->{favicon};
							$do_request = 0;
						} else {
							favicon_debug("But has no favicon");
							if (!favicon_checkdate(${$data->data}->{date}, $page_cache_time)) {
								$do_request = 0;
							}
						}
					}
					if ($do_request) {
						$favicon_additional->{url} = $temp;
						favicon_debug("<$favicon_additional->{url}> not found in PluginData");
						my $favicon_host = $favicon_additional->{url} . "favicon.ico";
						$res = favicon_request($favicon_host);

						# check to see if image is really a image
						if (favicon_validimage($res, $favicon_host)) {
							$favicon->{favicon} = $favicon_host;
							$favicon_additional->{favicon} = $favicon->{favicon};
						}
					}
				}

				# Save the data
				favicon_debug("Saving <$favicon->{url}> in PluginData");
				my $plugindata = MT::PluginData->load({plugin => "Favicon", key => $favicon->{url}});
				if (!$plugindata) {
					favicon_debug("Create");
					$plugindata = MT::PluginData->new;
					$plugindata->plugin("Favicon");
					$plugindata->key($favicon->{url});
				} else {
					favicon_debug("Update");
				}
				# update time
				$favicon->{date} = time;
				$plugindata->data(\$favicon);
				$plugindata->save or die $plugindata->errstr;

				if (exists $favicon_additional->{url} && $favicon->{url} ne $favicon_additional->{url}) {
					favicon_debug("Saving additional <$favicon_additional->{url}> in PluginData");
					my $plugindata_additional = MT::PluginData->load({plugin => "Favicon", key => $favicon_additional->{url}});
					if (!$plugindata_additional) {
						favicon_debug("Create");
						$plugindata_additional = MT::PluginData->new;
						$plugindata_additional->plugin("Favicon");
						$plugindata_additional->key($favicon_additional->{url});
					} else {
						favicon_debug("Update");
					}
					# update time
					$favicon_additional->{date} = time;
					$plugindata_additional->data(\$favicon_additional);
					$plugindata_additional->save or die $plugindata_additional->errstr;
				}
			}
		} else {
			favicon_debug("Error: URL <$url> with host <". $tmp_host . "> isn't valid");
		}
		favicon_debug("Returning <" . (exists $favicon->{favicon} ? $favicon->{favicon} : "") . "> for <" . (exists $favicon->{url} ? $favicon->{url} : "") . ">");
	} else {
		favicon_debug("No URL");
	}
	favicon_debug("---------------\n");
	return $favicon->{favicon} || "";
}

sub favicon_full {
	my($url, $site) = @_;
	return "<img src=\"$url\" alt=\"Favicon for $site\" title=\"Favicon for $site\" width=\"16\" height=\"16\" />";
}
